#pragma once
#include <asm/msr.h>

#define IA32_FEATURE_CONTROL_LOCK_BIT (1UL << 0)
#define IA32_FEATURE_CONTROL_VMXON_IN_SMX (1UL << 1)
#define IA32_FEATURE_CONTROL_VMXON_OUTSIDE_SMX (1UL << 2)
#define IA32_FEATURE_CONTROL_MSR (0x3a)

// NOTE: IA32_BASIC_VMX MSR, Appendix 1 p.4'586
// Bits 30:0 contain the 31-bit VMCS revision identifier
// Bit 31 is always 0
// Bit 44:32 report the number of bytes to allocate for the VMXON and VMCS
// regions
#define IA32_VMX_BASIC (0x480)

#define VMCS_REVISION_ID (__rdmsr(IA32_VMX_BASIC) & 0x7fffffff)
#define REGION_SIZE ((__rdmsr(IA32_VMX_BASIC) >> 32) & 0x1fff)

// NOTE: Appendix 8 p. 4'592 of Intel SDM
#define IA32_VMX_CR0_FIXED0 (0x486)
#define IA32_VMX_CR0_FIXED1 (0x487)
#define IA32_VMX_CR4_FIXED0 (0x488)
#define IA32_VMX_CR4_FIXED1 (0x489)

#define CR0_PE (1UL << 0)
#define CR0_NE (1UL << 5)
#define CR0_PG (1UL << 31)

bool ia32_feature_control_flags(void);

void patch_control_registers(void);
